@@ -124,6 +124,17 @@ class BluetoothTray(Gtk.Application):
     def on_statusicon_released(self, icon, x, y, button, time, position):
         if button == 3:
             menu = Gtk.Menu()
+
+            if not self.rfkill.hard_block:
+                if self.rfkill.soft_block:
+                    item = Gtk.MenuItem(label=_("Turn on Bluetooth"))
+                    item.connect("activate", self.turn_on_bluetooth)
+                    menu.append(item)
+                else:
+                    item = Gtk.MenuItem(label=_("Turn off Bluetooth"))
+                    item.connect("activate", self.turn_off_bluetooth)
+                    menu.append(item)
+
             if not(self.rfkill.hard_block or self.rfkill.soft_block):
                 item = Gtk.MenuItem(label=_("Send files to a device"))
                 item.connect("activate", self.send_files_cb)
@@ -135,7 +146,7 @@ class BluetoothTray(Gtk.Application):
 
             if len(self.paired_devices) > 0:
                 menu.append(Gtk.SeparatorMenuItem())
-                m_item = Gtk.MenuItem(_("Paired devices"))
+                m_item = Gtk.MenuItem(label=_("Paired devices"))
                 menu.append(m_item)
                 paired_menu = Gtk.Menu()
                 m_item.set_submenu(paired_menu)
@@ -157,21 +168,7 @@ class BluetoothTray(Gtk.Application):
             menu.append(item)
 
             menu.show_all()
-
-            if position == -1:
-                # The position and coordinates are unknown. This is the
-                # case when the XAppStatusIcon fallbacks as a Gtk.StatusIcon
-                menu.popup(None, None, None, None, button, time)
-            else:
-                def position_menu_cb(menu, pointer_x, pointer_y, user_data):
-                    [x, y, position] = user_data;
-                    if (position == Gtk.PositionType.BOTTOM):
-                        y = y - menu.get_allocation().height;
-                    if (position == Gtk.PositionType.RIGHT):
-                        x = x - menu.get_allocation().width;
-                    return (x, y, False)
-                device = Gdk.Display.get_default().get_device_manager().get_client_pointer()
-                menu.popup_for_device(device, None, None, position_menu_cb, [x, y, position], button, time)
+            icon.popup_menu(menu, x, y, button, time, position)
 
     def toggle_connect_cb(self, item, data = None):
         proxy = self.paired_devices[data]
@@ -184,6 +181,14 @@ class BluetoothTray(Gtk.Application):
     def open_manager_cb(self, item, data = None):
         subprocess.Popen(["blueberry"])
 
+    def turn_on_bluetooth(self, item):
+        self.rfkill.try_set_blocked(False)
+        return True
+
+    def turn_off_bluetooth(self,item):
+        self.rfkill.try_set_blocked(True)
+        return True
+
     def terminate(self, window = None, data = None):
         self.quit()
 
